"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getUpdateFee = getUpdateFee;
exports.fillPythUpdate = fillPythUpdate;
const hermes_client_1 = require("@pythnetwork/hermes-client");
const viem_1 = require("viem");
const multicall3_bundler_1 = require("./multicall3-bundler");
const pyth_abi_1 = require("./pyth-abi");
const debug_trace_call_1 = require("./tracer/debug-trace-call");
const trace_call_many_1 = require("./tracer/trace-call-many");
/**
 * Get the update fee for a given set of update data
 *
 * @param client - The public client instance
 * @param pythContractAddress - The Pyth contract address
 * @param updateData - Array of hex-encoded update data
 * @returns Promise resolving to the update fee in wei
 */
async function getUpdateFee(client, pythContractAddress, updateData) {
    return await client.readContract({
        address: pythContractAddress,
        abi: pyth_abi_1.IPythAbi,
        functionName: "getUpdateFee",
        args: [updateData],
    });
}
/**
 * Fill the Pyth data for a given call request.
 * Requires a client that supports trace_callMany or debug_traceCall with a bundler.
 *
 * @param client - The public client instance
 * @param call - The call request to fill with Pyth data
 * @param pythContractAddress - The Pyth contract address
 * @param hermesEndpoint - The Hermes endpoint URL for fetching price updates
 * @param config - Configuration options for tracing and bundling. Can be either:
 *   - `DebugTraceCallConfig`: For debug_traceCall method with a bundler function to combine Pyth update with original call.
 *     The bundler creates a single transaction that executes both the Pyth update and the original call.
 *   - `TraceCallManyConfig`: For trace_callMany method which traces multiple calls separately.
 *     This method traces the Pyth update and original call as separate transactions.
 * @returns Promise resolving to Pyth update object or undefined if no Pyth data needed
 */
async function fillPythUpdate(client, call, pythContractAddress, hermesEndpoint, config) {
    const defaultConfig = {
        method: "debug_traceCall",
        bundler: multicall3_bundler_1.multicall3Bundler,
        maxIter: 5,
    };
    const finalConfig = config ?? defaultConfig;
    const traceActionsClient = client
        .extend(debug_trace_call_1.debugTraceCallAction)
        .extend(trace_call_many_1.traceCallManyAction);
    const hermesClient = new hermes_client_1.HermesClient(hermesEndpoint);
    let requiredPriceFeeds = new Set();
    let pythUpdate;
    for (let i = 0; i < finalConfig.maxIter; i++) {
        let priceFeeds = new Set();
        if (finalConfig.method === "debug_traceCall") {
            const bundledCall = pythUpdate
                ? finalConfig.bundler(pythUpdate, call)
                : call;
            const traceResult = await traceActionsClient.debugTraceCall(bundledCall);
            priceFeeds = (0, debug_trace_call_1.extractPythPriceFeedsFromDebugTraceCall)(traceResult, pythContractAddress);
        }
        else {
            const calls = pythUpdate ? [pythUpdate.call, call] : [call];
            const traceResult = await traceActionsClient.traceCallMany(calls);
            priceFeeds = (0, trace_call_many_1.extractPythPriceFeedsFromTraceCallMany)(traceResult, pythContractAddress);
        }
        const oldSize = requiredPriceFeeds.size;
        requiredPriceFeeds = new Set([...requiredPriceFeeds, ...priceFeeds]);
        if (oldSize === requiredPriceFeeds.size) {
            break;
        }
        const hermesResponse = await hermesClient.getLatestPriceUpdates([
            ...requiredPriceFeeds,
        ]);
        const updateData = hermesResponse.binary.data.map((data) => ("0x" + data));
        const updateFee = await getUpdateFee(client, pythContractAddress, updateData);
        pythUpdate = {
            call: {
                to: pythContractAddress,
                data: (0, viem_1.encodeFunctionData)({
                    abi: pyth_abi_1.IPythAbi,
                    functionName: "updatePriceFeeds",
                    args: [updateData],
                }),
                from: call.from,
                value: updateFee,
            },
            updateData,
            updateFee,
        };
    }
    return pythUpdate;
}
